/* 
 *
 * PDFlib PLOP DS:
 * Demonstrate digital signature features with optional signature visualization
 *
 */

#include <iostream>
#include <string>

#include "plop.hpp"

using namespace std;
using namespace pdflib;

namespace
{
wstring get_wstring(const PLOP& plop, const string& utf8_string);
}

int
main(int argc, char **argv)
{
    PLOP plop;

    /* This is where input files live. Adjust as necessary. */
    const wstring searchpath(L"../data");

    /* Set to true to enable signature visualization */
    const bool visualize = false;
    const wstring visfile(L"signing_man_pdfa1a.pdf");

    if (argc < 3)
    {
	wcerr << L"usage: sign <infile> <outfile>" << endl;
	return 2;
    }

    try
    {
        wostringstream optlist;
        int visdoc;

	optlist << L"searchpath={" << searchpath << L"}";

	/* Basic signature options */
	const wstring sign_opts =
            L"ltv=try "          /* use ltv=full to ensure LTV status */
#if defined(USE_PKCS11)
            L"engine=pkcs#11 digitalid={filename=cryptoki.dll} "
#else
            L"engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "
#endif
            ;

	plop.set_option(optlist.str());

        /*
         * Caution: For simplicity we assume that the program arguments are
         * encoded as UTF-8, which might not be true in all cases!
         */
        const wstring doc_name(get_wstring(plop, string(argv[1])));
        const int doc = plop.open_document(doc_name, L"");
	if (doc == -1)
	{
	    wcerr << L"Error: " << plop.get_apiname()
                        << L": " << plop.get_errmsg() << endl;
	    return 2;
	}

        /* Basic signature options */
	optlist.str(L"");
        optlist << sign_opts;

        if (visualize)
        {
            if ((visdoc = plop.open_document(visfile, L"")) == -1)
            {
                wcerr << L"Error: " << plop.get_apiname()
                            << L": " << plop.get_errmsg() << endl;
                return 2;
            }

            /* Add field option for visualization */
            optlist << L"field={visdoc="
                    << visdoc << L" rect={50 50 adapt adapt}} ";
        }

	if (plop.prepare_signature(optlist.str()) == -1)
	{
	    wcerr << L"Error: " << plop.get_apiname()
                        << L": " << plop.get_errmsg() << endl;
	    return 2;
	}

	optlist.str(L"");
	optlist << L"input=" << doc;
        const wstring out_name(get_wstring(plop, string(argv[2])));
	if (plop.create_document(out_name, optlist.str()) == -1)
	{
	    wcerr << L"Error: " << plop.get_apiname()
                        << L": " << plop.get_errmsg() << endl;
	    return 2;
	}

	plop.close_document(doc, L"");

        if (visualize)
        {
            plop.close_document(visdoc, L"");
        }
    }
    catch (PLOP::Exception &ex)
    {
	wcerr << L"PLOP exception occurred in encrypt:" << endl;
	wcerr << L"[" << ex.get_errnum() << L"] " << ex.get_apiname()
		<< L": " << ex.get_errmsg() << endl;
	return 2;
    }

    return 0;
}

namespace
{

/*
 * Get a wstring for the given string, assuming the string is encoded in
 * UTF-8.
 */
wstring get_wstring(const PLOP& plop, const string& utf8_string)
{
    const size_t size = sizeof(wstring::value_type);
    string wide_string;

    switch (size)
    {
    case 2:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf16");
        break;

    case 4:
        wide_string = plop.convert_to_unicode(L"utf8",
                            utf8_string, L"outputformat=utf32");
        break;

    default:
        throw std::logic_error("Unsupported wchar_t size");
    }

    return wstring(reinterpret_cast<const wchar_t *>(wide_string.data()),
            wide_string.length() / size);
}

} // end anonymous namespace

