#!/usr/bin/env ruby
#
# PDFlib PLOP DS:
# Demonstrate mass signature with optional PKCS#11 session handling
#

require './PLOP'

# create a new PLOP object
plop = PLOP.new

# Basic signature option

sign_opts = 
        "ltv=try " +        #* use ltv=full to ensure LTV status */
	#when using PKCS11
        # "engine=pkcs#11 digitalid={filename=cryptoki.dll} "
        "engine=builtin digitalid={filename=demo_signer_rsa_2048.p12} password={demo} "



# This is where input files live. Adjust as necessary.
searchpath = "../data"

# Set to 1 to enable PKCS#11 session handling
PKCS11 = 0

begin
    if (ARGV.length < 1)
	raise "usage: sign <infile> ...\n"
    end
    plop.set_option("searchpath=" + searchpath)

    if (plop.prepare_signature(sign_opts) == -1)
		raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
    end

    0.step(ARGV.length-1, 1) do |i|
	printf("Signing '%s' ...\n", ARGV[i]);

	if ((doc = plop.open_document(ARGV[i], "")) == -1)
	    printf( "ERROR: %s: %s\n", plop.get_apiname(), plop.get_errmsg())
	    next
	end
	signed_name = sprintf("%s-signed.pdf", ARGV[i]);
	    
	if (plop.create_document(signed_name, "input=#{doc}") == -1)
		    raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
	    if (PKCS11 == 1)
		# The PKCS#11 session may have expired;
		# restart and try again
		if (plop.prepare_signature(sign_opts) == -1)
			    raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
		end

		if (plop.create_document(signed_name, "input=#{doc}") == -1)
			    raise "ERROR: " + plop.get_apiname() + ": " + plop.get_errmsg()
		end
	    end
		
	end
	plop.close_document(doc, "")
    end

rescue PLOPException => pe
    print pe.backtrace.join("\n")
    printf("PLOP exception occurred in multisign:\n")
    print "[" + pe.get_errnum.to_s + "] " + pe.get_apiname + \
                        ": " + pe.get_errmsg + "\n"
rescue Exception => e
    print e.backtrace.join("\n") + "\n" + e.to_s + "\n"
ensure
    plop.delete() if plop
end
